/*
 *  This file is part of x48, an emulator of the HP-48sx Calculator.
 *  Copyright (C) 1994  Eddie C. Dost  (ecd@dressler.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Log: hp48.h,v $
 * Revision 1.5  1994/09/18  15:31:58  ecd
 * started Real Time support
 *
 * Revision 1.4  1994/09/13  16:58:42  ecd
 * changed to plain X11
 *
 * Revision 1.3  1994/08/31  18:25:23  ecd
 * added read_nibble_crc
 *
 * Revision 1.2  1994/08/27  11:29:48  ecd
 * changed keyboard interrupt handling.
 *
 * Revision 1.1  1994/08/26  11:09:18  ecd
 * Initial revision
 *
 *
 *
 * $Id: hp48.h,v 1.5 1994/09/18 15:31:58 ecd Exp ecd $
 */

#ifndef _HP48_H
#define _HP48_H 1

#include "global.h"

#include <sys/time.h>

#define ROM_SIZE   0x80000
#define RAM_SIZE   0x10000
#define PORT1_SIZE 0x40000
#define PORT2_SIZE 0x40000

#define P_FIELD          0
#define WP_FIELD         1
#define XS_FIELD         2
#define X_FIELD          3
#define S_FIELD          4
#define M_FIELD          5
#define B_FIELD          6
#define W_FIELD          7
#define A_FIELD         15
#define IN_FIELD        16
#define OUT_FIELD       17
#define OUTS_FIELD      18

#define NR_CONFIG	 8
#define NR_RSTK		 8
#define NR_PSTAT	16

typedef int  flag_t;
typedef int  word_4;
typedef long word_20;

typedef struct keystate_t {
  int rows[9];
} keystate_t;

typedef struct display_t {

  int  on;

  long disp_start;
  long disp_end;

  int  offset;
  int  lines;
  int  skip;

  int  contrast;

  long menu_start;
  long menu_end;

  int  annunc;

} display_t;

typedef struct saturn_t {

  unsigned char A[16], B[16], C[16], D[16];

  word_20       d[2];

#define D0	d[0]
#define D1	d[1]

  word_4        P;
  word_20       PC;

  unsigned char R0[16], R1[16], R2[16], R3[16], R4[16];
  unsigned char IN[4];
  unsigned char OUT[3];

  flag_t        CARRY;

  unsigned char PSTAT[NR_PSTAT];
  unsigned char XM, SB, SR, MP;

  unsigned char hexmode;

  word_20	rstk[NR_RSTK];
  short         rstkp;

  keystate_t    keybuf;

  unsigned char intenable;
  unsigned char int_pending;
  unsigned char kbd_ien;

  long		configs[NR_CONFIG];
  short         daisy_state;

  long		ram32k;

  /* Devices: Address 0x100 - 0x13f */
  long		devices;

  unsigned char disp_io;

  unsigned char contrast_ctrl;
  unsigned char disp_test;

  unsigned int  crc;

  unsigned char power_status;
  unsigned char power_ctrl;

  unsigned char mode;

  unsigned char annunc;

  unsigned char baud;

  unsigned char card_ctrl;
  unsigned char card_status;

  unsigned char io_ctrl;
  unsigned char rcs;
  unsigned char tcs;

  unsigned char rbr;
  unsigned char tbr;

  unsigned char sreq;

  unsigned char ir_ctrl;

  unsigned char base_off;

  unsigned char lcr;
  unsigned char lbr;

  unsigned char scratch;

  unsigned char base_nibble;

  long          disp_addr;
  long          line_offset;
  long          line_count;

  long          unknown;

  unsigned char t1_ctrl;
  unsigned char t2_ctrl;

  long          menu_addr;

  long          unknown2;

  int           timer1;
  long          timer2;

  long		t1_instr;
  long		t2_instr;

  unsigned char *rom;
  unsigned char *ram;
  unsigned char *port1;
  unsigned char *port2;

} saturn_t;

#define NIBBLES_PER_ROW 0x22

extern int		got_alarm;

extern int		set_t1;
extern long		set_t2;
extern struct timeval	tv_t1;
extern struct timeval	tv_t2;

#define timerdiff(tvp1, tvp2) \
        (((tvp1)->tv_sec - (tvp2)->tv_sec) * 1000000 + \
        ((tvp1)->tv_usec - (tvp2)->tv_usec))

extern display_t	display;
extern void		init_display __ProtoType__((void));

extern saturn_t		saturn;

extern int		exit_emulator __ProtoType__((void));
extern int		init_emulator __ProtoType__((void));

extern int		serial_init __ProtoType__((void));
extern void		transmit_char __ProtoType__((void));
extern void		receive_char __ProtoType__((void));

extern void		do_kbd_int __ProtoType__((void));

extern void		write_nibble __ProtoType__((long addr, int val));
extern int		read_nibble __ProtoType__((long addr));
extern int		read_nibble_crc __ProtoType__((long addr));

extern int		emulate	__ProtoType__((void));


#endif /* !_HP48_H */
