/*
 *  This file is part of x48, an emulator of the HP-48sx Calculator.
 *  Copyright (C) 1994  Eddie C. Dost  (ecd@dressler.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Log: lcd.c,v $
 * Revision 1.4  1994/09/18  15:29:22  ecd
 * turned off unused rcsid message
 *
 * Revision 1.3  1994/09/13  16:57:00  ecd
 * changed to plain X11
 *
 * Revision 1.2  1994/08/31  18:23:21  ecd
 * changed display initialization.
 *
 * Revision 1.1  1994/08/26  11:09:02  ecd
 * Initial revision
 *
 * $Id: lcd.c,v 1.4 1994/09/18 15:29:22 ecd Exp ecd $
 */


#include "global.h"

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "hp48.h"
#include "hp48_emu.h"
#include "x48_x11.h"
#include "annunc.h"

#define ANN_BATTERY	 0x88
#define ANN_ALPHA	 0x84
#define ANN_RIGHT	 0x82
#define ANN_LEFT	 0x81
#define ANN_IO		 0xa0
#define ANN_BUSY	 0x90

#define LCD_ROWS	       64
#define ROWS	 	       70
#define LCD_COLUMNS	      132
#define COLUMNS		      140

display_t display;

typedef struct row_t {
  unsigned char bits[NIBBLES_PER_ROW];
} row_t;

row_t lcd[LCD_ROWS];
row_t old_lcd[LCD_ROWS];

char lcd_buffer[ROWS][COLUMNS];

void
#ifdef __FunctionProto__
init_display(void)
#else
init_display()
#endif
{
  int i, j;

  display.on = (saturn.disp_io & 0x8) >> 3;

  display.disp_start = saturn.disp_addr & 0xffffe;
  display.offset = saturn.disp_io & 0x7;
  display.lines = saturn.line_count & 0x3f;
  display.skip = saturn.line_offset;
  display.disp_end = display.disp_start +
		((NIBBLES_PER_ROW + display.skip) * (display.lines + 1));

  display.menu_start = saturn.menu_addr;
  display.menu_end = saturn.menu_addr + 0x110;

  display.contrast = saturn.contrast_ctrl;

  display.annunc = saturn.annunc;

  memset(old_lcd, 0, sizeof(lcd));
  memset(lcd, 0, sizeof(lcd));
  for (i = 0; i < ROWS; i++) {
    for (j = 0; j < COLUMNS; j++) {
      lcd_buffer[i][j] = 0;
    }
  }
}

void
#ifdef __FunctionProto__
draw_nibble(int x, int y, int val)
#else
draw_nibble(x, y, val)
int x;
int y;
int val;
#endif
{
  int i, b, c, r, nib;
  XPoint pt[4 * 4];
  XPoint *ptp = pt;

  if (y >= LCD_ROWS || x * 4 >= LCD_COLUMNS)
    return;

  c = x * 4;
  r = y;
  b = 1;

  x *= 4 * 2;
  x += 4;
  y *= 2;
  y += 20;

  nib = lcd_buffer[r][c];
  for (i = 0; i < 4; i++, val >>= 1) {
    if (val & 1) {
      if ((nib & b) == 0) {
        nib |= b;
        ptp[0].x = x;   ptp[0].y = y;
        ptp[1].x = x+1; ptp[1].y = y;
        ptp[2].x = x;   ptp[2].y = y+1;
        ptp[3].x = x+1; ptp[3].y = y+1;
        ptp += 4;
      }
    } else {
      if (nib & b) {
        nib &= ~b;
        XSetForeground(dpy, disp.gc, COLOR(LCD));
        XFillRectangle(dpy, disp.map, disp.gc, x, y, 2, 2);
        XClearArea(dpy, disp.win, x, y, 2, 2, False);
        XSetForeground(dpy, disp.gc, COLOR(PIXEL));
      }
    }
    b <<= 1;
    x += 2;
    if (x > 264)
      break;
  }
  lcd_buffer[r][c] = nib;
  if (ptp != pt) {
    XDrawPoints(dpy, disp.map, disp.gc, pt, ptp - pt, CoordModeOrigin);
    XDrawPoints(dpy, disp.win, disp.gc, pt, ptp - pt, CoordModeOrigin);
  }
}

void
#ifdef __FunctionProto__
draw_row(long addr, int row)
#else
draw_row(addr, row)
long addr;
int row;
#endif
{
  int i, v;

  for (i = 0; i < NIBBLES_PER_ROW; i++) {
    lcd[row].bits[i] = read_nibble(addr + i);
  }
  for (i = 0; i < NIBBLES_PER_ROW; i++) {
    v = lcd[row].bits[i];
    if (v != old_lcd[row].bits[i]) {
      draw_nibble(i, row, v & 0x0f);
    }
  }
}

void
#ifdef __FunctionProto__
update_display(void)
#else
update_display()
#endif
{
  int i, j;
  long addr;

  if (display.on) {
    memcpy(old_lcd, lcd, sizeof(lcd));
    addr = display.disp_start;
    for (i = 0; i <= display.lines; i++) {
      draw_row(addr, i);
      addr += (NIBBLES_PER_ROW + display.skip) & 0xfff;
    }
    if (i < LCD_ROWS) {
      addr = display.menu_start;
      for (; i < LCD_ROWS; i++) {
        draw_row(addr, i);
        addr += NIBBLES_PER_ROW;
      }
    }
  } else {
    memset(lcd, 0, sizeof(lcd));
    for (i = 0; i < 64; i++) {
      for (j = 0; j < NIBBLES_PER_ROW; j++) {
        draw_nibble(j, i, 0x00);
      }
    }
  }
}

void
#ifdef __FunctionProto__
disp_draw_nibble_by_addr(long addr)
#else
disp_draw_nibble_by_addr(addr)
long addr;
#endif
{
  long offset;
  int x, y;
  unsigned char val;

  offset = (addr - display.disp_start);
  x = offset % (NIBBLES_PER_ROW + display.skip);
  y = offset / (NIBBLES_PER_ROW + display.skip);
  if (y < 0 || y > 63 || x < 0 || x > 33)
    return;
  val = read_nibble(addr);
  if (val != lcd[y].bits[x]) {
    old_lcd[y].bits[x] = lcd[y].bits[x];
    lcd[y].bits[x] = val;
    draw_nibble(x, y, val);
  }
}

void
#ifdef __FunctionProto__
menu_draw_nibble_by_addr(long addr)
#else
menu_draw_nibble_by_addr(addr)
long addr;
#endif
{
  long offset;
  int x, y;
  unsigned char val;

  offset = (addr - display.menu_start);
  x = offset % NIBBLES_PER_ROW;
  y = display.lines + (offset / NIBBLES_PER_ROW) + 1;
  if (y < 0 || y > 63 || x < 0 || x > 33)
    return;
  val = read_nibble(addr);
  if (val != lcd[y].bits[x]) {
    old_lcd[y].bits[x] = lcd[y].bits[x];
    lcd[y].bits[x] = val;
    draw_nibble(x, y, val);
  }
}

struct ann_struct {
  int    bit;
  int    x;
  int    y;
  int    width;
  int    height;
  char  *bits;
  Pixmap pixmap;
} ann_tbl[] = {
  { ANN_LEFT, 16, 4, ann_left_width, ann_left_height, ann_left_bits },
  { ANN_RIGHT, 61, 4, ann_right_width, ann_right_height, ann_right_bits },
  { ANN_ALPHA, 106, 4, ann_alpha_width, ann_alpha_height, ann_alpha_bits },
  { ANN_BATTERY, 151, 4, ann_battery_width, ann_battery_height,
                         ann_battery_bits },
  { ANN_BUSY, 196, 4, ann_busy_width, ann_busy_height, ann_busy_bits },
  { ANN_IO, 241, 4, ann_io_width, ann_io_height, ann_io_bits },
  { 0 }
};

void
#ifdef __FunctionProto__
draw_annunc(void)
#else
draw_annunc()
#endif
{
  int val;
  int i;
  static int last_annunc_state = -1;

  val = display.annunc;
  if (val == last_annunc_state)
    return;
  last_annunc_state = val;
  for (i = 0; ann_tbl[i].bit; i++) {
    if ((ann_tbl[i].bit & val) == ann_tbl[i].bit) {
      XCopyPlane(dpy, ann_tbl[i].pixmap, disp.map, disp.gc, 0, 0,
                 ann_tbl[i].width, ann_tbl[i].height,
                 ann_tbl[i].x, ann_tbl[i].y, 1);
      XCopyPlane(dpy, ann_tbl[i].pixmap, disp.win, disp.gc, 0, 0,
                 ann_tbl[i].width, ann_tbl[i].height,
                 ann_tbl[i].x, ann_tbl[i].y, 1);
    } else {
      XSetForeground(dpy, disp.gc, COLOR(LCD));
      XFillRectangle(dpy, disp.map, disp.gc, ann_tbl[i].x, ann_tbl[i].y,
                     ann_tbl[i].width, ann_tbl[i].height);
      XClearArea(dpy, disp.win, ann_tbl[i].x, ann_tbl[i].y,
                 ann_tbl[i].width, ann_tbl[i].height, False);
      XSetForeground(dpy, disp.gc, COLOR(PIXEL));
    }
  }
}

void
#ifdef __FunctionProto__
init_annunc(void)
#else
init_annunc()
#endif
{
  int i;
 
  for (i = 0; ann_tbl[i].bit; i++) {
    ann_tbl[i].pixmap = XCreateBitmapFromData(dpy, disp.map, ann_tbl[i].bits,
                                ann_tbl[i].width, ann_tbl[i].height);
  }
}

